# Module `Hamiltonians`

This module contains definitions of Hamiltonians, in particular specific
physical models of interest. These are organised by means of an interface
around the abstract type [`AbstractHamiltonian`](@ref), in the spirit of the
`AbstractArray` interface as discussed in the [Julia Documentation](https://docs.julialang.org/en/v1/manual/interfaces/).

```@docs
Hamiltonians
```

## Usage with FCIQMC and exact diagonalisation

In order to define a specific model Hamiltonian with relevant parameters
for the model, instantiate the model like this in the input file:

```julia-repl
hubb = HubbardReal1D(BoseFS((1,2,0,3)); u=1.0, t=1.0)
```

The Hamiltonian `hubb` is now ready to be used for FCIQMC in [`lomc!`](@ref) 
and for exact diagolisation with [`KrylovKit.jl`](https://github.com/Jutho/KrylovKit.jl) directly, or after 
transforming into a sparse matrix first with 
```julia-repl
using SparseArrays
sh = sparse(hubb)
```
or into a full matrix with
```julia-repl
using LinearAlgebra
fh = Matrix(hubb)
```
This functionality relies on 
```@docs
BasisSetRep
sparse
Matrix
```

## Model Hamiltonians

Here is a list of fully implemented model Hamiltonians. So far there are two variants
implemented of the one-dimensional Bose-Hubbard model real space as well as a momentum-space
Hubbard chain.

```@docs
HubbardReal1D
ExtendedHubbardReal1D
HubbardMom1D
Transcorrelated1D
BoseHubbardReal1D2C
BoseHubbardMom1D2C
HubbardRealSpace
MatrixHamiltonian
```

## Hamiltonian Wrappers
The following Hamiltonians are constructed from an existing 
Hamiltonian instance and change its behaviour:
```@docs
GutzwillerSampling
GuidingVectorSampling
ParitySymmetry
TimeReversalSymmetry
```

## Observables
Observables are [`AbstractHamiltonian`](@ref)s that represent a physical 
observable. Their ground state expectation values can be sampled by passing
them into [`AllOverlaps`](@ref).
```@docs
G2MomCorrelator
G2RealCorrelator
Momentum
```

## Hamiltonians interface

Behind the implementation of a particular model is a more abstract interface for defining
hamiltonians. If you want to define a new model you should make use of this interface. The
most general form of a model Hamiltonian should subtype to `AbstractHamiltonian` and
implement the relevant methods.

```@docs
AbstractHamiltonian
offdiagonals
diagonal_element
starting_address
```

The following functions may be implemented instead of [`offdiagonals`](@ref).

```@docs
num_offdiagonals
get_offdiagonal
```

The following functions come with default implementations, but may be customized.

```@docs
random_offdiagonal
Hamiltonians.LOStructure
dimension
```

## Geometry

```@docs
LatticeGeometry
PeriodicBoundaries
HardwallBoundaries
LadderBoundaries
num_neighbours
neighbour_site
```
